package gs.smolban.model.users

import cats.syntax.all.*

/** Describes a _Scope_, or service of Smolban, to which a user can have access.
  *
  * Scopes consist of two parts:
  *
  *   - `service`: The name of a service Smolban offers.
  *   - `name`: The name of the scope within the given service.
  *
  * @param service
  *   The name of a service Smolban offers (e.g. `api`, `ui`)
  * @param name
  *   The name of the scope within the service.
  */
sealed abstract class Scope(
  val service: String,
  val name: String
)

object Scope:

  given CanEqual[Scope, Scope] = CanEqual.derived

  object Services:

    val Api: String = "api"
    val Ui: String  = "ui"

  end Services

  object Names:

    val Global: String = "global"
    val Group: String  = "group"

  end Names

  val Delimiter: String = ":"

  /** The `api:global` scope covers the Smolban API across all groups.
    */
  case object ApiGlobal extends Scope(Services.Api, Names.Global):
    override def toString(): String = s"[$service$Delimiter$name]"

  /** The `api:group:<id>` scope refers the the Smolban API for a specific
    * [[Group]].
    *
    * @param groupId
    *   The unique identifier of the [[Group]].
    */
  case class ApiGroup(
    groupId: gs.smolban.model.Group.Id
  ) extends Scope(Services.Api, Names.Group):

    override def toString(): String =
      s"[$service$Delimiter$name$Delimiter${groupId.show}]"

  /** The `ui:global` scope covers the Smolban UI across all groups.
    */
  case object UiGlobal extends Scope(Services.Ui, Names.Global):
    override def toString(): String = s"[$service$Delimiter$name]"

  /** The `ui:group:<id>` scope refers the the Smolban UI for a specific
    * [[Group]].
    *
    * @param groupId
    *   The unique identifier of the [[Group]].
    */
  case class UiGroup(
    groupId: gs.smolban.model.Group.Id
  ) extends Scope(Services.Ui, Names.Group):

    override def toString(): String =
      s"[$service$Delimiter$name$Delimiter${groupId.show}]"

end Scope
